module.exports="SymbolTypes:\n    VAR: &var 0\n    CONSTRUCTOR: &constructor 1\n    FUNC: &func 2\n# Internal patterns to save typing\n__type: &__type\n    id: null\n    callable: *var\n    args: null\n    type: null\n    attr: {}\n    template: null\n    argsTemplate: null\n__func: &__func\n    callable: *func\n    args: []\n    attr: {}\n    template: null\n    argsTemplate: null\n\n# C# Templates\nTemplates:\n    # Syntax\n    EqualitySyntaxTemplate: &EqualitySyntaxTemplate !!js/function >\n        (lhs, op, rhs) => {\n            if (op.includes('!')) {\n                return `${lhs} != ${rhs}`;\n            }\n            else {\n                return `${lhs} == ${rhs}`;\n            }\n        }\n    EosSyntaxTemplate: &EosSyntaxTemplate null\n    EofTemplate: &EofSyntaxTemplate null\n    # BSON Object Type templates\n    CodeTypeTemplate: &CodeTypeTemplate null\n    StringTypeTemplate: &StringTypeTemplate !!js/function >\n        (str) => {\n            let newStr = str;\n            if (\n              (str.charAt(0) === '\\'' && str.charAt(str.length - 1) === '\\'') ||\n              (str.charAt(0) === '\"' && str.charAt(str.length - 1) === '\"')) {\n              newStr = str.substr(1, str.length - 2);\n            }\n            return `\"${newStr.replace(/\\\\([\\s\\S])|(\")/g, '\\\\$1$2')}\"`;\n        }\n    RegexTypeTemplate: &RegexTypeTemplate !!js/function >\n        (pattern, flags) => {\n            flags = flags === '' ? '' : `(?${flags})`;\n            // Double-quote stringify\n            const str = flags + pattern;\n            let newStr = str;\n            if (\n                (str.charAt(0) === '\\'' && str.charAt(str.length - 1) === '\\'') ||\n                (str.charAt(0) === '\"' && str.charAt(str.length - 1) === '\"')) {\n                    newStr = str.substr(1, str.length - 2);\n            }\n            return `new Regex(\"${newStr.replace(/\\\\([\\s\\S])|(\")/g, '\\\\$1$2')}\")`;\n        }\n    BoolTypeTemplate: &BoolTypeTemplate null\n    IntegerTypeTemplate: &IntegerTypeTemplate null\n    DecimalTypeTemplate: &DecimalTypeTemplate null\n    LongBasicTypeTemplate: &LongBasicTypeTemplate null\n    HexTypeTemplate: &HexTypeTemplate null\n    OctalTypeTemplate: &OctalTypeTemplate !!js/function >\n        (literal) => {\n            literal = literal.replace(/[oO]+/g, '0')\n            return parseInt(literal, 8).toString()\n        }\n    NumericTypeTemplate: &NumericTypeTemplate null\n    ArrayTypeTemplate: &ArrayTypeTemplate !!js/function >\n        (literal, depth) => {\n            const indent = '\\n' + '    '.repeat(depth);\n            const initialIndent = '\\n' + '    '.repeat(depth-1);\n            if (literal === '') {\n              return 'new BsonArray()'\n            }\n\n            return `new BsonArray${initialIndent}{${literal}${initialIndent}}`;\n        }\n    ArrayTypeArgsTemplate: &ArrayTypeArgsTemplate !!js/function >\n        (arg, depth, last) => {\n            const indent = '\\n' + '    '.repeat(depth);\n            const arr = arg.split(', new').join(`, ${indent}new`)\n\n            return last ? `${indent}${arr}` : `${indent}${arr},`;\n        }\n    NullTypeTemplate: &NullTypeTemplate !!js/function >\n        () => {\n            return 'BsonNull.Value';\n        }\n    UndefinedTypeTemplate: &UndefinedTypeTemplate !!js/function >\n        () => {\n            return 'BsonUndefined.Value';\n        }\n    ObjectTypeTemplate: &ObjectTypeTemplate !!js/function >\n        (literal) => {\n            if (literal === '') {\n                return `new BsonDocument()`;\n            }\n            return literal;\n        }\n    ObjectTypeArgsTemplate: &ObjectTypeArgsTemplate !!js/function >\n        (arg, depth) => {\n            const indent = '\\n' + '    '.repeat(depth);\n            const initialIndent = '\\n' + '    '.repeat(depth-1);\n            const doubleStringify = (str) => {\n                let newStr = str;\n                if (\n                  (str.charAt(0) === '\\'' && str.charAt(str.length - 1) === '\\'') ||\n                  (str.charAt(0) === '\"' && str.charAt(str.length - 1) === '\"')) {\n                    newStr = str.substr(1, str.length - 2);\n                }\n                return `\"${newStr.replace(/\\\\([\\s\\S])|(\")/g, '\\\\$1$2')}\"`;\n            }\n\n            if (arg.length === 1) {\n              return `new BsonDocument(${doubleStringify(arg[0][0])}, ${arg[0][1]})`;\n            }\n\n            const pairs = arg.map((pair) => {\n                return `${indent}{ ${doubleStringify(pair[0])}, ${pair[1]} }`;\n            }).join(', ');\n\n            return `new BsonDocument${initialIndent}{${pairs}${initialIndent}}`\n        }\n    # BSON Object Method templates\n    CodeCodeTemplate: &CodeCodeTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.Code`;\n        }\n    CodeCodeArgsTemplate: &CodeCodeArgsTemplate null\n    CodeScopeTemplate: &CodeScopeTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.Scope`;\n        }\n    CodeScopeArgsTemplate: &CodeScopeArgsTemplate null\n    ObjectIdToStringTemplate: &ObjectIdToStringTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.ToString`;\n        }\n    ObjectIdToStringArgsTemplate: &ObjectIdToStringArgsTemplate null\n    ObjectIdEqualsTemplate: &ObjectIdEqualsTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.Equals`;\n        }\n    ObjectIdEqualsArgsTemplate: &ObjectIdEqualsArgsTemplate !!js/function >\n        (lhs, arg) => {\n            if (arg.indexOf('new') === 0) {\n              arg = arg.replace(/new /g, '')\n            }\n            return `(new ${arg})`;\n        }\n\n    ObjectIdGetTimestampTemplate: &ObjectIdGetTimestampTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.Timestamp`;\n        }\n    ObjectIdGetTimestampArgsTemplate: &ObjectIdGetTimestampArgsTemplate !!js/function >\n        (lhs) => {\n            return '';\n        }\n    BinaryValueTemplate: &BinaryValueTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.ToString`;\n        }\n    BinaryValueArgsTemplate: &BinaryValueArgsTemplate null\n    BinaryLengthTemplate: &BinaryLengthTemplate !!js/function >\n        () => {\n          return '';\n        }\n    BinaryLengthArgsTemplate: &BinaryLengthArgsTemplate !!js/function >\n        (lhs) => {\n          return `${lhs}.Bytes.Length`;\n        }\n    BinaryToStringTemplate: &BinaryToStringTemplate !!js/function >\n        (lhs) => {\n          return `${lhs}.ToString`;\n        }\n    BinaryToStringArgsTemplate: &BinaryToStringArgsTemplate null\n    BinarySubtypeTemplate: &BinarySubtypeTemplate !!js/function >\n        (lhs) => {\n          return `${lhs}.SubType`;\n        }\n    BinarySubtypeArgsTemplate: &BinarySubtypeArgsTemplate !!js/function >\n        () => {\n          return '';\n        }\n    DBRefGetDBTemplate: &DBRefGetDBTemplate !!js/function >\n        (lhs) => {\n          return `${lhs}.DatabaseName`;\n        }\n    DBRefGetDBArgsTemplate: &DBRefGetDBArgsTemplate !!js/function >\n        () => {\n          return '';\n        }\n    DBRefGetCollectionTemplate: &DBRefGetCollectionTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.CollectionName`;\n        }\n    DBRefGetCollectionArgsTemplate: &DBRefGetCollectionArgsTemplate !!js/function >\n        () => {\n          return '';\n        }\n    DBRefGetIdTemplate: &DBRefGetIdTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.Id`;\n        }\n    DBRefGetIdArgsTemplate: &DBRefGetIdArgsTemplate !!js/function >\n        () => {\n          return '';\n        }\n    DBRefToStringTemplate: &DBRefToStringTemplate !!js/function >\n        (lhs) => {\n          return `${lhs}.ToString`;\n        }\n    DBRefToStringArgsTemplate: &DBRefToStringArgsTemplate null\n    DoubleValueOfTemplate: &DoubleValueOfTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}`;\n        }\n    DoubleValueOfArgsTemplate: &DoubleValueOfArgsTemplate !!js/function >\n        (lhs) => {\n            return '';\n        }\n    Int32ValueOfTemplate: &Int32ValueOfTemplate !!js/function >\n        (lhs) => {\n            return lhs;\n        }\n    Int32ValueOfArgsTemplate: &Int32ValueOfArgsTemplate !!js/function >\n        (lhs) => {\n            return '';\n        }\n    Int32ToStringTemplate: &Int32ToStringTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.ToString`;\n        }\n    Int32ToStringArgsTemplate: &Int32ToStringArgsTemplate null\n    LongEqualsTemplate: &LongEqualsTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} == `;\n        }\n    LongEqualsArgsTemplate: &LongEqualsArgsTemplate !!js/function >\n        (lhs, arg) => {\n              return `${arg}`;\n        }\n    LongToIntTemplate: &LongToIntTemplate !!js/function >\n        (lhs) => {\n            return `(int) ${lhs}`;\n        }\n    LongToIntArgsTemplate: &LongToIntArgsTemplate !!js/function >\n        () => {\n            return '';\n        }\n    LongValueOfTemplate: &LongValueOfTemplate !!js/function >\n        (lhs) => {\n            return `(int) ${lhs}`;\n        }\n    LongValueOfArgsTemplate: &LongValueOfArgsTemplate !!js/function >\n        () => {\n            return '';\n        }\n    LongToStringTemplate: &LongToStringTemplate !!js/function > # Also has emit method\n        (lhs) => {\n            return lhs;\n        }\n    LongToStringArgsTemplate: &LongToStringArgsTemplate null # Also has emit method\n    LongToNumberTemplate: &LongToNumberTemplate !!js/function >\n        (lhs) => {\n            return `(double) ${lhs}`;\n        }\n    LongToNumberArgsTemplate: &LongToNumberArgsTemplate !!js/function >\n        () => {\n            return '';\n        }\n    LongAddTemplate: &LongAddTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} +`;\n        }\n    LongAddArgsTemplate: &LongAddArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongSubtractTemplate: &LongSubtractTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} -`;\n        }\n    LongSubtractArgsTemplate: &LongSubtractArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongMultiplyTemplate: &LongMultiplyTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} *`;\n        }\n    LongMultiplyArgsTemplate: &LongMultiplyArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongDivTemplate: &LongDivTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} /`;\n        }\n    LongDivArgsTemplate: &LongDivArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongModuloTemplate: &LongModuloTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} %`;\n        }\n    LongModuloArgsTemplate: &LongModuloArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongAndTemplate: &LongAndTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} &`;\n        }\n    LongAndArgsTemplate: &LongAndArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongOrTemplate: &LongOrTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} |`;\n        }\n    LongOrArgsTemplate: &LongOrArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongXorTemplate: &LongXorTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} ^`;\n        }\n    LongXorArgsTemplate: &LongXorArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongShiftLeftTemplate: &LongShiftLeftTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} <<`;\n        }\n    LongShiftLeftArgsTemplate: &LongShiftLeftArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongShiftRightTemplate: &LongShiftRightTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} >>`;\n        }\n    LongShiftRightArgsTemplate: &LongShiftRightArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongCompareTemplate: &LongCompareTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} - `;\n        }\n    LongCompareArgsTemplate: &LongCompareArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return arg;\n        }\n    LongIsOddTemplate: &LongIsOddTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}`;\n        }\n    LongIsOddArgsTemplate: &LongIsOddArgsTemplate !!js/function >\n        () => {\n            return ' % 2 == 1';\n        }\n    LongIsZeroTemplate: &LongIsZeroTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} == 0`;\n        }\n    LongIsZeroArgsTemplate: &LongIsZeroArgsTemplate !!js/function >\n        () => {\n            return '';\n        }\n    LongIsNegativeTemplate: &LongIsNegativeTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} < 0`;\n        }\n    LongIsNegativeArgsTemplate: &LongIsNegativeArgsTemplate !!js/function >\n        () => {\n            return '';\n        }\n    LongNegateTemplate: &LongNegateTemplate !!js/function >\n        () => {\n            return '-';\n        }\n    LongNegateArgsTemplate: &LongNegateArgsTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}`;\n        }\n    LongNotTemplate: &LongNotTemplate !!js/function >\n        () => {\n            return '~';\n        }\n    LongNotArgsTemplate: &LongNotArgsTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}`;\n        }\n    LongNotEqualsTemplate: &LongNotEqualsTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} !=`;\n        }\n    LongNotEqualsArgsTemplate: &LongNotEqualsArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongGreaterThanTemplate: &LongGreaterThanTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} >`;\n        }\n    LongGreaterThanArgsTemplate: &LongGreaterThanArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongGreaterThanOrEqualTemplate: &LongGreaterThanOrEqualTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} >=`;\n        }\n    LongGreaterThanOrEqualArgsTemplate: &LongGreaterThanOrEqualArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongLessThanTemplate: &LongLessThanTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} <`;\n        }\n    LongLessThanArgsTemplate: &LongLessThanArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongLessThanOrEqualTemplate: &LongLessThanOrEqualTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} <=`;\n        }\n    LongLessThanOrEqualArgsTemplate: &LongLessThanOrEqualArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return ` ${arg}`;\n        }\n    LongFloatApproxTemplate: &LongFloatApproxTemplate !!js/function >\n        (lhs) => {\n            return `(float) ${lhs}`;\n        }\n    LongTopTemplate: &LongTopTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} >> 32`;\n        }\n    LongBottomTemplate: &LongBottomTemplate !!js/function >\n        (lhs) => {\n           return `${lhs} & 0x0000ffff`;\n        }\n    TimestampToStringTemplate: &TimestampToStringTemplate !!js/function >\n        (lhs) => {\n          return `${lhs}.ToString`;\n        }\n    TimestampToStringArgsTemplate: &TimestampToStringArgsTemplate null\n    TimestampEqualsTemplate: &TimestampEqualsTemplate !!js/function >\n        (lhs) => {\n          return `${lhs}.Equals`;\n        }\n    TimestampEqualsArgsTemplate: &TimestampEqualsArgsTemplate null\n    TimestampGetLowBitsTemplate: &TimestampGetLowBitsTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.ToUniversalTime`;\n        }\n    TimestampGetLowBitsArgsTemplate: &TimestampGetLowBitsArgsTemplate null\n    TimestampGetHighBitsTemplate: &TimestampGetHighBitsTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.Increment`;\n        }\n    TimestampGetHighBitsArgsTemplate: &TimestampGetHighBitsArgsTemplate !!js/function >\n        (lhs) => {\n            return '';\n        }\n    TimestampTTemplate: &TimestampTTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.ToUniversalTime()`;\n        }\n    TimestampITemplate: &TimestampITemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.Increment`;\n        }\n    TimestampCompareTemplate: &TimestampCompareTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.CompareTo`;\n        }\n    TimestampCompareArgsTemplate: &TimestampCompareArgsTemplate null\n    TimestampNotEqualsTemplate: &TimestampNotEqualsTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} != `;\n        }\n    TimestampNotEqualsArgsTemplate: &TimestampNotEqualsArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return `${arg}`;\n        }\n    TimestampGreaterThanTemplate: &TimestampGreaterThanTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} > `;\n        }\n    TimestampGreaterThanArgsTemplate: &TimestampGreaterThanArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return `${arg}`;\n        }\n    TimestampGreaterThanOrEqualTemplate: &TimestampGreaterThanOrEqualTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} >= `;\n        }\n    TimestampGreaterThanOrEqualArgsTemplate: &TimestampGreaterThanOrEqualArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return `${arg}`;\n        }\n    TimestampLessThanTemplate: &TimestampLessThanTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} < `;\n        }\n    TimestampLessThanArgsTemplate: &TimestampLessThanArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return `${arg}`;\n        }\n    TimestampLessThanOrEqualTemplate: &TimestampLessThanOrEqualTemplate !!js/function >\n        (lhs) => {\n            return `${lhs} <= `;\n        }\n    TimestampLessThanOrEqualArgsTemplate: &TimestampLessThanOrEqualArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return `${arg}`;\n        }\n    SymbolValueOfTemplate: &SymbolValueOfTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}`;\n        }\n    SymbolValueOfArgsTemplate: &SymbolValueOfArgsTemplate !!js/function >\n        () => {\n            return '';\n        }\n    SymbolInspectTemplate: &SymbolInspectTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}`;\n        }\n    SymbolInspectArgsTemplate: &SymbolInspectArgsTemplate !!js/function >\n        () => {\n            return '';\n        }\n    SymbolToStringTemplate: &SymbolToStringTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}`;\n        }\n    SymbolToStringArgsTemplate: &SymbolToStringArgsTemplate !!js/function >\n        () => {\n            return '';\n        }\n    # Symbol Templates\n    CodeSymbolTemplate: &CodeSymbolTemplate  !!js/function > # Also has process method\n        () => {\n            return 'BsonJavaScript';\n        }\n    CodeSymbolArgsTemplate: &CodeSymbolArgsTemplate !!js/function >\n        (lhs, code, scope) => {\n            // Double quote stringify\n            let newStr = code === undefined ? '' : code;\n            const str = newStr;\n            if (\n                (str.charAt(0) === '\\'' && str.charAt(str.length - 1) === '\\'') ||\n                (str.charAt(0) === '\"' && str.charAt(str.length - 1) === '\"')) {\n                newStr = str.substr(1, str.length - 2);\n            }\n            code = `\"${newStr.replace(/\\\\([\\s\\S])|(\")/g, '\\\\$1$2')}\"`;\n            return (scope === undefined) ? `(${code})` : `WithScope(${code}, ${scope})`;\n        }\n    ObjectIdSymbolTemplate: &ObjectIdSymbolTemplate !!js/function >\n        () => {\n            return 'ObjectId';\n        }\n    ObjectIdSymbolArgsTemplate: &ObjectIdSymbolArgsTemplate !!js/function >\n        (lhs, str) => {\n            let newStr = str;\n            if (\n                (str.charAt(0) === '\\'' && str.charAt(str.length - 1) === '\\'') ||\n                (str.charAt(0) === '\"' && str.charAt(str.length - 1) === '\"')) {\n                newStr = str.substr(1, str.length - 2);\n            }\n            return `(\"${newStr.replace(/\\\\([\\s\\S])|(\")/g, '\\\\$1$2')}\")`;\n        }\n    BinarySymbolTemplate: &BinarySymbolTemplate !!js/function >\n        () => {\n            return 'BsonBinaryData';\n        }\n    BinarySymbolArgsTemplate: &BinarySymbolArgsTemplate !!js/function >\n        (lhs, bytes, type) => {\n            const str = bytes;\n            let newStr = str;\n            if (\n              (str.charAt(0) === '\\'' && str.charAt(str.length - 1) === '\\'') ||\n              (str.charAt(0) === '\"' && str.charAt(str.length - 1) === '\"')) {\n                newStr = str.substr(1, str.length - 2);\n            }\n            bytes = `\"${newStr.replace(/\\\\([\\s\\S])|(\")/g, '\\\\$1$2')}\"`;\n\n            if (type === null) {\n                return `(System.Text.Encoding.ASCII.GetBytes(${bytes}))`;\n            }\n            return `(System.Text.Encoding.ASCII.GetBytes(${bytes}), ${type})`;\n        }\n    BinarySymbolSubtypeDefaultTemplate: &BinarySymbolSubtypeDefaultTemplate !!js/function >\n        () => {\n            return 'BsonBinarySubType.Binary';\n        }\n    BinarySymbolSubtypeFunctionTemplate: &BinarySymbolSubtypeFunctionTemplate !!js/function >\n        () => {\n            return 'BsonBinarySubType.Function';\n        }\n    BinarySymbolSubtypeByteArrayTemplate: &BinarySymbolSubtypeByteArrayTemplate !!js/function >\n        () => {\n            return 'BsonBinarySubType.OldBinary';\n        }\n    BinarySymbolSubtypeUuidOldTemplate: &BinarySymbolSubtypeUuidOldTemplate !!js/function >\n        () => {\n            return 'BsonBinarySubType.UuidLegacy';\n        }\n    BinarySymbolSubtypeUuidTemplate: &BinarySymbolSubtypeUuidTemplate !!js/function >\n        () => {\n            return 'BsonBinarySubType.UuidStandard';\n        }\n    BinarySymbolSubtypeMd5Template: &BinarySymbolSubtypeMd5Template !!js/function >\n        () => {\n            return 'BsonBinarySubType.MD5';\n        }\n    BinarySymbolSubtypeUserDefinedTemplate: &BinarySymbolSubtypeUserDefinedTemplate !!js/function >\n        () => {\n            return 'BsonBinarySubType.UserDefined';\n        }\n    DBRefSymbolTemplate: &DBRefSymbolTemplate !!js/function >\n        () => {\n            return 'MongoDBRef';\n        }\n    DBRefSymbolArgsTemplate: &DBRefSymbolArgsTemplate null\n    DoubleSymbolTemplate: &DoubleSymbolTemplate !!js/function >\n        () => {\n            return '';\n        }\n    DoubleSymbolArgsTemplate: &DoubleSymbolArgsTemplate !!js/function >\n        (lhs, arg, type) => {\n            arg = arg === undefined ? 0 : arg;\n\n            if (type === '_hex') {\n              return `Convert.ToDouble(${arg})`;\n            }\n            if (type === '_string') {\n              return `Convert.ToDouble(${arg})`;\n            }\n\n            if (type === '_decimal') {\n              return arg;\n            }\n\n            return `${Math.round(arg).toFixed(1)}`;\n        }\n    Int32SymbolTemplate: &Int32SymbolTemplate !!js/function >\n        () => {\n            return '';\n        }\n    Int32SymbolArgsTemplate: &Int32SymbolArgsTemplate !!js/function >\n        (lhs, arg, type) => {\n            arg = arg === undefined ? 0 : arg;\n\n            if (type === '_hex' || type === '_decimal' || type === '_string') {\n              return `Convert.ToInt32(${arg})`;\n            }\n\n            return arg;\n        }\n    LongSymbolTemplate: &LongSymbolTemplate !!js/function >\n        () => {\n            return '';\n        }\n    LongSymbolArgsTemplate: &LongSymbolArgsTemplate !!js/function > \n        (lhs, arg) => {\n            if (arg.indexOf('\\'') === 0 || arg.indexOf('\"') === 0) {\n              return `Convert.ToInt64(${arg})`;\n            }\n            return `${arg}L`;\n        }\n    LongSymbolMaxTemplate: &LongSymbolMaxTemplate !!js/function >\n        () => {\n            return 'Int64.MaxValue';\n        }\n    LongSymbolMaxArgsTemplate: &LongSymbolMaxArgsTemplate null\n    LongSymbolMinTemplate: &LongSymbolMinTemplate !!js/function >\n        () => {\n            return 'Int64.MinValue';\n        }\n    LongSymbolMinArgsTemplate: &LongSymbolMinArgsTemplate null\n    LongSymbolZeroTemplate: &LongSymbolZeroTemplate !!js/function >\n        (lhs) => {\n            return '0L';\n        }\n    LongSymbolZeroArgsTemplate: &LongSymbolZeroArgsTemplate null\n    LongSymbolOneTemplate: &LongSymbolOneTemplate !!js/function >\n        (lhs) => {\n            return '1L';\n        }\n    LongSymbolOneArgsTemplate: &LongSymbolOneArgsTemplate null\n    LongSymbolNegOneTemplate: &LongSymbolNegOneTemplate !!js/function >\n        (lhs) => {\n            return '-1L';\n        }\n    LongSymbolNegOneArgsTemplate: &LongSymbolNegOneArgsTemplate null\n    LongSymbolFromBitsTemplate: &LongSymbolFromBitsTemplate !!js/function > # Also has process method\n        () => {\n            return '';\n        }\n    LongSymbolFromBitsArgsTemplate: &LongSymbolFromBitsArgsTemplate null\n    LongSymbolFromIntTemplate: &LongSymbolFromIntTemplate !!js/function >\n        () => {\n            return '';\n        }\n    LongSymbolFromIntArgsTemplate: &LongSymbolFromIntArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return `${arg}L`;\n        }\n    LongSymbolFromNumberTemplate: &LongSymbolFromNumberTemplate null\n    LongSymbolFromNumberArgsTemplate: &LongSymbolFromNumberArgsTemplate null\n    LongSymbolFromStringTemplate: &LongSymbolFromStringTemplate !!js/function >\n        () => {\n            return 'Convert.ToInt64';\n        }\n    LongSymbolFromStringArgsTemplate: &LongSymbolFromStringArgsTemplate !!js/function >\n        (lhs, arg1, arg2) => {\n          if (arg2) {\n            return `(${arg1}, ${arg2})`\n          }\n          return `(${arg1}, 10)`\n        }\n    MinKeySymbolTemplate: &MinKeySymbolTemplate !!js/function > # Also has emit method\n        () => {\n            return 'BsonMinKey';\n        }\n    MinKeySymbolArgsTemplate: &MinKeySymbolArgsTemplate !!js/function > # Also has emit method\n        () => {\n            return '.Value';\n        }\n    MaxKeySymbolTemplate: &MaxKeySymbolTemplate !!js/function > # Also has emit method\n        () => {\n            return 'BsonMaxKey';\n        }\n    MaxKeySymbolArgsTemplate: &MaxKeySymbolArgsTemplate !!js/function > # Also has emit method\n        () => {\n            return '.Value';\n        }\n    TimestampSymbolTemplate: &TimestampSymbolTemplate !!js/function >\n        () => {\n            return 'BsonTimestamp';\n        }\n    TimestampSymbolArgsTemplate: &TimestampSymbolArgsTemplate !!js/function >\n        (lhs, arg1, arg2) => {\n            if (typeof arg1 === 'undefined') {\n                return '(0, 0)'\n            }\n            return `(${arg1}, ${arg2})`;\n        }\n    SymbolSymbolTemplate: &SymbolSymbolTemplate null # Has emit method \n    SymbolSymbolArgsTemplate: &SymbolSymbolArgsTemplate null # Has emit method\n    BSONRegExpSymbolTemplate: &BSONRegExpSymbolTemplate !!js/function >\n        () => {\n            return 'BsonRegularExpression';\n        }\n    BSONRegExpSymbolArgsTemplate: &BSONRegExpSymbolArgsTemplate !!js/function >\n        (lhs, pattern, flags) => {\n            return `(${pattern}${flags ? ', ' + flags : ''})`;\n        }\n    Decimal128SymbolTemplate: &Decimal128SymbolTemplate null\n    Decimal128SymbolArgsTemplate: &Decimal128SymbolArgsTemplate null\n    Decimal128SymbolFromStringTemplate: &Decimal128SymbolFromStringTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.Parse`;\n        }\n    Decimal128SymbolFromStringArgsTemplate: &Decimal128SymbolFromStringArgsTemplate !!js/function > \n        (lhs, arg) => {\n            return `(${arg})`;\n        }\n    Decimal128ToStringTemplate: &Decimal128ToStringTemplate !!js/function >\n        (lhs) => {\n            return `${lhs}.ToString`;\n        }\n    Decimal128ToStringArgsTemplate: &Decimal128ToStringArgsTemplate null\n    # BSON Util Templates\n    ObjectIdCreateFromHexStringTemplate: &ObjectIdCreateFromHexStringTemplate !!js/function >\n        (lhs) => {\n            return `new ${lhs}`;\n        }\n    ObjectIdCreateFromHexStringArgsTemplate: &ObjectIdCreateFromHexStringArgsTemplate null\n    ObjectIdCreateFromTimeTemplate: &ObjectIdCreateFromTimeTemplate !!js/function > # Also has emit method\n        (lhs) => {\n            return `new ${lhs}.GenerateNewId`;\n        }\n    ObjectIdCreateFromTimeArgsTemplate: &ObjectIdCreateFromTimeArgsTemplate !!js/function >\n        (lhs, arg) => {\n            if (!Number.isNaN(parseInt(arg, 10))) {\n              return `(Convert.ToInt32(${arg}))`;\n            }\n            return `(${arg})`;\n        }\n    ObjectIdIsValidTemplate: &ObjectIdIsValidTemplate !!js/function >\n        (lhs) => {\n            return `new ${lhs}`;\n        }\n    ObjectIdIsValidArgsTemplate: &ObjectIdIsValidArgsTemplate null\n    # JS Symbol Templates\n    ObjectSymbolTemplate: &ObjectSymbolTemplate null\n    ObjectSymbolArgsTemplate: &ObjectSymbolArgsTemplate null\n    ObjectSymbolCreateTemplate: &ObjectSymbolCreateTemplate !!js/function >\n        () => {\n            return '';\n        }\n    ObjectSymbolCreateArgsTemplate: &ObjectSymbolCreateArgsTemplate !!js/function >\n        (lhs, arg) => {\n            return arg;\n        }\n    NumberSymbolTemplate: &NumberSymbolTemplate !!js/function >\n        () => {\n            return '';\n        }\n    NumberSymbolArgsTemplate: &NumberSymbolArgsTemplate !!js/function >\n        (lhs, arg, type) => {\n            arg = arg === undefined ? 0 : arg;\n\n            if (type === '_string') {\n              if (arg.indexOf('.') >= 0) {\n                return `float.Parse(${arg})`\n              }\n              return `int.Parse(${arg})`;\n            }\n\n            return `(int) ${arg}`;\n        }\n    DateSymbolTemplate: &DateSymbolTemplate !!js/function > # Also has emit method\n        () => {\n            return 'Datetime';\n        }\n    DateSymbolArgsTemplate: &DateSymbolArgsTemplate null # Has emit method\n    DateSymbolNowTemplate: &DateSymbolNowTemplate null # Has emit method\n    DateSymbolNowArgsTemplate: &DateSymbolNowArgsTemplate null # Has emit method\n    RegExpSymbolTemplate: &RegExpSymbolTemplate !!js/function > # Also has process method\n        () => {\n            return 'Regex';\n        }\n    RegExpSymbolArgsTemplate: &RegExpSymbolArgsTemplate null\n    ImportTemplate: &ImportTemplate !!js/function >\n        (args) => {\n            const universal = ['using MongoDB.Bson;', 'using MongoDB.Driver;'];\n            const all = universal.concat(Object.values(args));\n            return all.join('\\n');\n        }\n    0ImportTemplate: &0ImportTemplate null\n    1ImportTemplate: &1ImportTemplate null\n    2ImportTemplate: &2ImportTemplate null\n    3ImportTemplate: &3ImportTemplate null\n    4ImportTemplate: &4ImportTemplate null\n    5ImportTemplate: &5ImportTemplate null\n    6ImportTemplate: &6ImportTemplate null\n    7ImportTemplate: &7ImportTemplate null\n    8ImportTemplate: &8ImportTemplate !!js/function >\n        () => {\n            return 'using System.Text.RegularExpressions;';\n        }\n    9ImportTemplate: &9ImportTemplate null\n    10ImportTemplate: &10ImportTemplate null\n    11ImportTemplate: &11ImportTemplate null\n    12ImportTemplate: &12ImportTemplate null\n    100ImportTemplate: &100ImportTemplate null\n    101ImportTemplate: &101ImportTemplate null\n    102ImportTemplate: &102ImportTemplate null\n    103ImportTemplate: &103ImportTemplate null\n    104ImportTemplate: &104ImportTemplate null\n    105ImportTemplate: &105ImportTemplate null\n    106ImportTemplate: &106ImportTemplate null\n    107ImportTemplate: &107ImportTemplate null\n    108ImportTemplate: &108ImportTemplate null\n    109ImportTemplate: &109ImportTemplate null\n    110ImportTemplate: &110ImportTemplate null\n    111ImportTemplate: &111ImportTemplate null\n    112ImportTemplate: &112ImportTemplate null\n    113ImportTemplate: &113ImportTemplate null\n    114ImportTemplate: &114ImportTemplate null\n    200ImportTemplate: &200ImportTemplate !!js/function >\n        () => {\n            return 'using System;';\n        }\n    201ImportTemplate: &201ImportTemplate null\n    300ImportTemplate: &300ImportTemplate null\n    301ImportTemplate: &301ImportTemplate null\n    302ImportTemplate: &302ImportTemplate null\n    303ImportTemplate: &303ImportTemplate null\n    304ImportTemplate: &304ImportTemplate null\n    305ImportTemplate: &305ImportTemplate null\n    306ImportTemplate: &306ImportTemplate null\n# Universal types\n# Everything inherits from StringType because we haven't implemented any of them.\nBasicTypes:\n    # Universal basic types\n    _bool: &BoolType\n        <<: *__type\n        id: \"_bool\"\n        code: 0\n        template: *BoolTypeTemplate\n    _integer: &IntegerType\n        <<: *__type\n        id: \"_integer\"\n        code: 1\n        template: *IntegerTypeTemplate\n    _long: &LongBasicType\n        <<: *__type\n        id: \"_long\"\n        code: 2\n        template: *LongBasicTypeTemplate\n    _decimal: &DecimalType\n        <<: *__type\n        id: \"_decimal\"\n        code: 3\n        template: *DecimalTypeTemplate\n    _hex: &HexType\n        <<: *__type\n        id: \"_hex\"\n        code: 4\n        template: *HexTypeTemplate\n    _octal: &OctalType\n        <<: *__type\n        id: \"_octal\"\n        code: 5\n        template: *OctalTypeTemplate\n    _numeric: &NumericType\n        <<: *__type\n        id: \"_numeric\"\n        code: 6\n        template: *NumericTypeTemplate\n    _string: &StringType\n        <<: *__type\n        id: \"_string\"\n        code: 7\n        template: *StringTypeTemplate\n    _regex: &RegexType\n        <<: *__type\n        id: \"_regex\"\n        code: 8\n        template: *RegexTypeTemplate\n    _array: &ArrayType\n        <<: *__type\n        id: \"_array\"\n        code: 9\n        template: *ArrayTypeTemplate\n        argsTemplate: *ArrayTypeArgsTemplate\n    _object: &ObjectType\n        <<: *__type\n        id: \"_object\"\n        code: 10\n        template: *ObjectTypeTemplate\n        argsTemplate: *ObjectTypeArgsTemplate\n    _null: &NullType\n        <<: *__type\n        id: \"_null\"\n        code: 11\n        template: *NullTypeTemplate\n    _undefined: &UndefinedType\n        <<: *__type\n        id: \"_undefined\"\n        code: 12\n        template: *UndefinedTypeTemplate\n\nSyntax:\n    equality: &EqualitySyntax\n        template: *EqualitySyntaxTemplate\n    eos: &EosSyntax\n        template: *EosSyntaxTemplate\n    eof: &EosSyntax\n        template: *EofSyntaxTemplate\nImports:\n    import:\n        template: *ImportTemplate\n    0:\n        template: *0ImportTemplate\n    1:\n        template: *1ImportTemplate\n    2:\n        template: *2ImportTemplate\n    3:\n        template: *3ImportTemplate\n    4:\n        template: *4ImportTemplate\n    5:\n        template: *5ImportTemplate\n    6:\n        template: *6ImportTemplate\n    7:\n        template: *7ImportTemplate\n    8:\n        template: *8ImportTemplate\n    9:\n        template: *9ImportTemplate\n    10:\n        template: *10ImportTemplate\n    11:\n        template: *11ImportTemplate\n    12:\n        template: *12ImportTemplate\n    100:\n        template: *100ImportTemplate\n    101:\n        template: *101ImportTemplate\n    102:\n        template: *102ImportTemplate\n    103:\n        template: *103ImportTemplate\n    104:\n        template: *104ImportTemplate\n    105:\n        template: *105ImportTemplate\n    106:\n        template: *106ImportTemplate\n    107:\n        template: *107ImportTemplate\n    108:\n        template: *108ImportTemplate\n    109:\n        template: *109ImportTemplate\n    110:\n        template: *110ImportTemplate\n    111:\n        template: *111ImportTemplate\n    112:\n        template: *112ImportTemplate\n    113:\n        template: *113ImportTemplate\n    114:\n        template: *114ImportTemplate\n    200:\n        template: *200ImportTemplate\n    201:\n        template: *201ImportTemplate\n    300:\n        template: *300ImportTemplate\n    301:\n        template: *301ImportTemplate\n    302:\n        template: *302ImportTemplate\n    303:\n        template: *303ImportTemplate\n    304:\n        template: *304ImportTemplate\n    305:\n        template: *305ImportTemplate\n    306:\n        template: *306ImportTemplate\nBsonTypes:\n    Code: &CodeType\n        <<: *__type\n        id: \"Code\"\n        code: 100\n        type: *ObjectType\n        attr:\n            code:\n                callable: *var\n                args: null\n                attr: null\n                id: \"code\"\n                type: *StringType\n                template: *CodeCodeTemplate\n                argsTemplate: *CodeCodeArgsTemplate\n            scope:\n                callable: *var\n                args: null\n                attr: null\n                id: \"scope\"\n                type: *StringType\n                template: *CodeScopeTemplate\n                argsTemplate: *CodeScopeArgsTemplate\n        template: *CodeTypeTemplate\n    ObjectId: &ObjectIdType\n        <<: *__type\n        id: \"ObjectId\"\n        code: 101\n        type: *ObjectType\n        attr:\n            toString:\n                <<: *__func\n                id: \"toString\"\n                type: *StringType\n                template: *ObjectIdToStringTemplate\n                argsTemplate: *ObjectIdToStringArgsTemplate\n            equals:\n                <<: *__func\n                id: \"equals\"\n                args:\n                    - [ \"ObjectId\" ]\n                type: *BoolType\n                template: *ObjectIdEqualsTemplate\n                argsTemplate: *ObjectIdEqualsArgsTemplate\n            getTimestamp:\n                <<: *__func\n                id: \"getTimestamp\"\n                type: *IntegerType\n                template: *ObjectIdGetTimestampTemplate\n                argsTemplate: *ObjectIdGetTimestampArgsTemplate\n    BinData: &BinaryType\n        <<: *__type\n        id: \"BinData\"\n        code: 102\n        type: *ObjectType\n        attr:\n            toString:\n                <<: *__func\n                id: \"toString\"\n                type: *StringType\n                template: *BinaryToStringTemplate\n                argsTemplate: *BinaryToStringArgsTemplate\n            base64:\n                <<: *__func\n                id: \"base64\"\n                type: *StringType\n                template: *BinaryValueTemplate\n                argsTemplate: *BinaryValueArgsTemplate\n            length:\n                <<: *__func\n                id: \"length\"\n                type: *IntegerType\n                template: *BinaryLengthTemplate\n                argsTemplate: *BinaryLengthArgsTemplate\n            subtype:\n                <<: *__func\n                id: \"subtype\"\n                type: *IntegerType\n                template: *BinarySubtypeTemplate\n                argsTemplate: *BinarySubtypeArgsTemplate\n    DBRef: &DBRefType\n        <<: *__type\n        id: \"DBRef\"\n        code: 103\n        type: *ObjectType\n        attr:\n            toString:\n                <<: *__func\n                id: \"toString\"\n                type: *StringType\n                template: *DBRefToStringTemplate\n                argsTemplate: *DBRefToStringArgsTemplate\n            getDb:\n                <<: *__func\n                id: \"getDb\"\n                type: *StringType\n                template: *DBRefGetDBTemplate\n                argsTemplate: *DBRefGetDBArgsTemplate\n            $db:\n                callable: *var\n                args: null\n                attr: null\n                id: \"$db\"\n                type: *StringType\n                template: *DBRefGetDBTemplate\n                argsTemplate: *DBRefGetDBArgsTemplate\n            getCollection:\n                <<: *__func\n                id: \"getCollection\"\n                type: *StringType\n                template: *DBRefGetCollectionTemplate\n                argsTemplate: *DBRefGetCollectionArgsTemplate\n            getRef:\n                <<: *__func\n                id: \"getRef\"\n                type: *StringType\n                template: *DBRefGetCollectionTemplate\n                argsTemplate: *DBRefGetCollectionArgsTemplate\n            $ref:\n                callable: *var\n                args: null\n                attr: null\n                id: \"$ref\"\n                type: *StringType\n                template: *DBRefGetCollectionTemplate\n                argsTemplate: *DBRefGetCollectionArgsTemplate\n            getId:\n                <<: *__func\n                id: \"getId\"\n                type: *StringType\n                template: *DBRefGetIdTemplate\n                argsTemplate: *DBRefGetIdArgsTemplate\n            $id:\n                callable: *var\n                args: null\n                attr: null\n                id: \"$id\"\n                type: *StringType\n                template: *DBRefGetIdTemplate\n                argsTemplate: *DBRefGetIdArgsTemplate\n    NumberInt: &Int32Type\n        <<: *__type\n        id: \"NumberInt\"\n        code: 105\n        type: *ObjectType\n        attr:\n            toString:\n                <<: *__func\n                id: \"toString\"\n                type: *StringType\n                template: *Int32ToStringTemplate\n                argsTemplate: *Int32ToStringArgsTemplate\n            valueOf: # TODO: needed?\n                <<: *__func\n                id: \"valueOf\"\n                type: *Int32Type\n                template: *Int32ValueOfTemplate\n                argsTemplate: *Int32ValueOfArgsTemplate\n    NumberLong: &LongType\n        <<: *__type\n        id: \"NumberLong\"\n        code: 106\n        type: *ObjectType\n        attr:\n            toString:\n                <<: *__func\n                id: \"LongtoString\" # Needs process method\n                type: *StringType\n            valueOf:\n                <<: *__func\n                id: \"valueOf\"\n                type: *LongBasicType\n                template: *LongValueOfTemplate\n                argsTemplate: *LongValueOfArgsTemplate\n            top:\n                callable: *var\n                args: null\n                attr: null\n                id: \"top\"\n                type: *IntegerType\n                template: *LongTopTemplate\n                argsTemplate: null\n            bottom:\n                callable: *var\n                args: null\n                attr: null\n                id: \"bottom\"\n                type: *IntegerType\n                template: *LongBottomTemplate\n                argsTemplate: null\n            floatApprox:\n                callable: *var\n                args: null\n                attr: null\n                id: \"floatApprox\"\n                type: *IntegerType\n                template: *LongFloatApproxTemplate\n                argsTemplate: null\n    MinKeyType: &MinKeyType\n        <<: *__type\n        id: \"MinKey\"\n        code: 107\n        type: *ObjectType\n    MaxKeyType: &MaxKeyType\n        <<: *__type\n        id: \"MaxKey\"\n        code: 108\n        type: *ObjectType\n    Timestamp: &TimestampType\n        <<: *__type\n        id: \"TimestampFromShell\"\n        code: 110\n        type: *ObjectType\n        attr:\n            toString:\n                <<: *__func\n                id: \"toString\"\n                type: *StringType\n                template: *TimestampToStringTemplate\n                argsTemplate: *TimestampToStringArgsTemplate\n            getTime:\n                <<: *__func\n                id: \"getTime\"\n                type: *IntegerType\n                template: *TimestampGetLowBitsTemplate\n                argsTemplate: *TimestampGetLowBitsArgsTemplate\n            getInc:\n                <<: *__func\n                id: \"getInc\"\n                type: *IntegerType\n                template: *TimestampGetHighBitsTemplate\n                argsTemplate: *TimestampGetHighBitsArgsTemplate\n            t:\n                callable: *var\n                args: null\n                attr: null\n                id: \"getTime\"\n                type: *IntegerType\n                template: *TimestampTTemplate\n                argsTemplate: null\n            i:\n                callable: *var\n                args: null\n                attr: null\n                id: \"getInc\"\n                type: *IntegerType\n                template: *TimestampITemplate\n                argsTemplate: null\n    Symbol: &SymbolType\n        <<: *__type\n        id: \"Symbol\"\n        code: 111\n        type: *ObjectType\n    NumberDecimal: &Decimal128Type\n        <<: *__type\n        id: \"NumberDecimal\"\n        code: 112\n        type: *ObjectType\n        attr:\n            toString:\n                <<: *__func\n                id: \"toString\"\n                type: *StringType\n                template: *Decimal128ToStringTemplate\n                argsTemplate: *Decimal128ToStringArgsTemplate\n    SUBTYPE_DEFAULT:\n        id: \"SUBTYPE_DEFAULT\"\n        callable: *var\n        args: null\n        code: 113\n        type: *IntegerType\n        template: *BinarySymbolSubtypeDefaultTemplate\n    SUBTYPE_FUNCTION:\n        id: \"SUBTYPE_FUNCTION\"\n        callable: *var\n        args: null\n        code: 113\n        type: *IntegerType\n        template: *BinarySymbolSubtypeFunctionTemplate\n    SUBTYPE_BYTE_ARRAY:\n        id: \"SUBTYPE_BYTE_ARRAY\"\n        callable: *var\n        args: null\n        code: 113\n        type: *IntegerType\n        template: *BinarySymbolSubtypeByteArrayTemplate\n    SUBTYPE_UUID_OLD:\n        id: \"SUBTYPE_UUID_OLD\"\n        callable: *var\n        args: null\n        code: 113\n        type: *IntegerType\n        template: *BinarySymbolSubtypeUuidOldTemplate\n    SUBTYPE_UUID:\n        id: \"SUBTYPE_UUID\"\n        callable: *var\n        args: null\n        code: 113\n        type: *IntegerType\n        template: *BinarySymbolSubtypeUuidTemplate\n    SUBTYPE_MD5:\n        id: \"SUBTYPE_MD5\"\n        callable: *var\n        args: null\n        code: 113\n        type: *IntegerType\n        template: *BinarySymbolSubtypeMd5Template\n    SUBTYPE_USER_DEFINED:\n        id: \"SUBTYPE_USER_DEFINED\"\n        callable: *var\n        args: null\n        code: 113\n        type: *IntegerType\n        template: *BinarySymbolSubtypeUserDefinedTemplate\nJSTypes:\n    Date: &DateType\n        <<: *__type\n        id: \"Date\"\n        code: 200\n        type: *ObjectType\n        attr: {} # TODO: no built-in date methods added yet\n    RegExp: &RegExpType\n        <<: *__type\n        id: \"RegExp\"\n        code: 8\n        type: *ObjectType\n        attr: {}\n\n\n\n\nBsonSymbols:\n    Code: &CodeSymbol\n        id: \"Code\"\n        code: 100\n        callable: *constructor\n        args:\n            - [ *StringType, null ]\n            - [ *ObjectType, null ]\n        type: *CodeType\n        attr: {}\n        template: *CodeSymbolTemplate\n        argsTemplate: *CodeSymbolArgsTemplate\n    ObjectId: &ObjectIdSymbol\n        id: \"ObjectId\"\n        code: 101\n        callable: *constructor\n        args:\n            - [ *StringType, null ]\n        type: *ObjectIdType\n        attr:\n            fromDate:\n                <<: *__func\n                id: \"ObjectIdCreateFromTime\"\n                args:\n                    - [ *DateType ]\n                type: *ObjectIdType\n                template: *ObjectIdCreateFromTimeTemplate\n                argsTemplate: *ObjectIdCreateFromTimeArgsTemplate\n        template: *ObjectIdSymbolTemplate\n        argsTemplate: *ObjectIdSymbolArgsTemplate\n    BinData: &BinarySymbol\n        id: \"BinData\"\n        code: 102\n        callable: *constructor\n        args:\n            - [ *IntegerType ]\n            - [ *StringType ]\n        type: *BinaryType\n        attr: {}\n        template: *BinarySymbolTemplate\n        argsTemplate: *BinarySymbolArgsTemplate\n    DBRef:\n        id: \"DBRef\"\n        code: 103\n        callable: *constructor\n        args:\n            - [ *StringType ]\n            - [ *ObjectIdType ]\n            - [ *StringType, null ]\n        type: *DBRefType\n        attr: {}\n        template: *DBRefSymbolTemplate\n        argsTemplate: *DBRefSymbolArgsTemplate\n    NumberInt:\n        id: \"Int32\"\n        code: 105\n        callable: *constructor\n        args:\n            - [ *IntegerType, *StringType, null ]\n        type: *Int32Type\n        attr: {}\n        template: *Int32SymbolTemplate\n        argsTemplate: *Int32SymbolArgsTemplate\n    NumberLong:\n        id: \"NumberLong\"\n        code: 106\n        callable: *constructor\n        args:\n            - [ *IntegerType, *StringType, null ]\n        type: *LongType\n        attr: {}\n        template: *LongSymbolTemplate\n        argsTemplate: *LongSymbolArgsTemplate\n    MinKey:\n        id: \"MinKey\"\n        code: 107\n        callable: *constructor\n        args: []\n        type: *MinKeyType\n        attr: {}\n        template: *MinKeySymbolTemplate\n        argsTemplate: *MinKeySymbolArgsTemplate\n    MaxKey:\n        id: \"MaxKey\"\n        code: 108\n        callable: *constructor\n        args: []\n        type: *MaxKeyType\n        attr: {}\n        template: *MaxKeySymbolTemplate\n        argsTemplate: *MaxKeySymbolArgsTemplate\n    Timestamp:\n        id: \"Timestamp\"\n        code: 110\n        callable: *constructor\n        args:\n            - [ *IntegerType, null ]\n            - [ *IntegerType, null ]\n        type: *TimestampType\n        attr: {}\n        template: *TimestampSymbolTemplate\n        argsTemplate: *TimestampSymbolArgsTemplate\n    Symbol:\n        id: \"Symbol\"\n        code: 111\n        callable: *constructor\n        args:\n            - [ *StringType ]\n        type: *SymbolType\n        attr: {}\n        template: *SymbolSymbolTemplate\n        argsTemplate: *SymbolSymbolArgsTemplate\n    NumberDecimal:\n        id: \"NumberDecimal\"\n        code: 112\n        callable: *constructor\n        args:\n            - [ *StringType, *NumericType, null ]\n        type: *Decimal128Type\n        attr: {}\n        template: *Decimal128SymbolTemplate\n        argsTemplate: *Decimal128SymbolArgsTemplate\n\nJSSymbols:\n    Object:\n        id: \"Object\"\n        code: 10\n        callable: *var\n        args: null\n        type: *ObjectType\n        attr:\n            create:\n                id: \"create\"\n                callable: *func\n                args:\n                    - [ *ObjectType ]\n                type: *ObjectType\n                attr: {}\n                template: *ObjectSymbolCreateTemplate\n                argsTemplate: *ObjectSymbolCreateArgsTemplate\n        template: *ObjectSymbolTemplate\n        argsTemplate: *ObjectSymbolArgsTemplate\n    Number:\n        id: \"Number\"\n        code: 2\n        callable: *constructor\n        args:\n            - [ *IntegerType, *StringType, null ]\n        type: *NumericType\n        attr: {} # TODO: no built-in number funcs added yet\n        template: *NumberSymbolTemplate\n        argsTemplate: *NumberSymbolArgsTemplate\n    Date: # Needs emit method\n        id: \"Date\"\n        code: 200\n        callable: *constructor\n        args:\n            - [ *StringType, *NumericType, null ]\n            - [ *NumericType, null ]\n            - [ *NumericType, null ]\n            - [ *NumericType, null ]\n            - [ *NumericType, null ]\n            - [ *NumericType, null ]\n            - [ *NumericType, null ]\n        type: *DateType\n        attr: # TODO: add more date funcs?\n            now:\n                id: \"now\"\n                callable: *constructor\n                args: []\n                type: *DateType\n                attr: {}\n                template: *DateSymbolNowTemplate\n                argsTemplate: *DateSymbolNowArgsTemplate\n        template: *DateSymbolTemplate\n        argsTemplate: *DateSymbolArgsTemplate\n    ISODate: # Needs emit method\n        id: \"ISODate\"\n        code: 200\n        callable: *constructor\n        args:\n            - [ *StringType, null ]\n        type: *DateType\n        attr:\n            now:\n                id: \"now\"\n                callable: *constructor\n                args: []\n                type: *DateType\n                attr: {}\n                template: *DateSymbolNowTemplate\n                argsTemplate: *DateSymbolNowArgsTemplate\n        template: *DateSymbolTemplate\n        argsTemplate: *DateSymbolArgsTemplate\n    RegExp: # Needs process method\n        id: \"RegExp\"\n        code: 8\n        callable: *constructor\n        args:\n            - [ *StringType, *RegexType ]\n            - [ *StringType, null ]\n        type: *RegExpType\n        attr: {} # TODO: no built-in regex funcs added yet\n        template: *RegExpSymbolTemplate\n        argsTemplate: *RegExpSymbolArgsTemplate\n\n";
